package com.bpcl.lpg.core.service.impl;

import org.slf4j.LoggerFactory;
import org.apache.commons.lang.StringUtils;
import java.util.Objects;
import com.bpcl.lpg.facades.data.LpgDedupRequestData;
import java.text.MessageFormat;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.net.http.HttpResponse;
import com.bpcl.lpg.facades.data.LpgRequestData;
import com.bpcl.lpg.core.service.BpclHttpClientService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.service.LpgConnectionService;

public class LpgConnectionServiceImpl implements LpgConnectionService
{
    private static final Logger LOG;
    private ConfigurationService configurationService;
    private BpclHttpClientService httpClientService;
    
    public HttpResponse<String> getResponse(final LpgRequestData lpgRequestData, final String requestId, final String endPointName) throws JsonProcessingException {
        this.validateParameter(lpgRequestData, endPointName);
        final String endPoint = this.getEndPoint(endPointName, requestId);
        final String jsonBody = this.getRequestJson(lpgRequestData, endPoint);
        return this.getHttpClientService().executePostMethod(endPoint, jsonBody, endPointName);
    }
    
    private String getRequestJson(final LpgRequestData lpgRequestData, final String endPointName) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        if (lpgRequestData.getEndPointName().equalsIgnoreCase("deduplicationUrl")) {
            final LpgDedupRequestData lpgDedupRequestData = this.getDedupRequestData(lpgRequestData);
            if (LpgConnectionServiceImpl.LOG.isDebugEnabled()) {
                LpgConnectionServiceImpl.LOG.debug(MessageFormat.format("Input json :: {0} for end point :: {1} ", objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(lpgDedupRequestData), endPointName));
            }
            return objectMapper.writeValueAsString(lpgDedupRequestData);
        }
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(lpgRequestData);
        return objectMapper.writeValueAsString(lpgRequestData);
    }
    
    private LpgDedupRequestData getDedupRequestData(final LpgRequestData lpgRequestData) {
        final LpgDedupRequestData lpgDedupRequestData = new LpgDedupRequestData();
        lpgDedupRequestData.setPartnerCode(lpgRequestData.getPartnerCode());
        lpgDedupRequestData.setRequestId(lpgRequestData.getRequestId());
        lpgDedupRequestData.setAddl1(lpgRequestData.getAddl1());
        lpgDedupRequestData.setAddl2(lpgRequestData.getAddl2());
        lpgDedupRequestData.setApiRequestNo(lpgRequestData.getApiRequestNo());
        lpgDedupRequestData.setHashValue(lpgRequestData.getHashValue());
        lpgDedupRequestData.setMobileNo(lpgRequestData.getMobileNo());
        lpgDedupRequestData.setEmailId(lpgRequestData.getEmailId());
        lpgDedupRequestData.setAadhaarNo(lpgRequestData.getAadhaarNo());
        lpgDedupRequestData.setBankAccountNo(lpgRequestData.getBankAccountNo());
        lpgDedupRequestData.setBankIFSC(lpgRequestData.getBankIFSC());
        lpgDedupRequestData.setRationCardStateCd(lpgRequestData.getRationCardStateCd());
        lpgDedupRequestData.setRationCardNo(lpgRequestData.getRationCardNo());
        lpgDedupRequestData.setHouseRationCardNo(lpgRequestData.getHouseRationCardNo());
        lpgDedupRequestData.setTypeOfDedup(lpgRequestData.getTypeOfDedup());
        lpgDedupRequestData.setAhlTin(lpgRequestData.getAhlTin());
        return lpgDedupRequestData;
    }
    
    private void validateParameter(final LpgRequestData lpgRequestData, final String endPointName) {
        Objects.requireNonNull(lpgRequestData, "LPG Request data should not be null");
        Objects.requireNonNull(endPointName, "End point name should not be null");
    }
    
    private String getEndPoint(final String endPointName, final String requestId) {
        final String endpointUri = this.getConfigurationService().getConfiguration().getString(endPointName);
        Objects.requireNonNull(endpointUri, "Endpoint should not be null");
        return StringUtils.isBlank(requestId) ? endpointUri : MessageFormat.format(endpointUri, requestId);
    }
    
    public ConfigurationService getConfigurationService() {
        return this.configurationService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public BpclHttpClientService getHttpClientService() {
        return this.httpClientService;
    }
    
    public void setHttpClientService(final BpclHttpClientService httpClientService) {
        this.httpClientService = httpClientService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgConnectionServiceImpl.class);
    }
}
